# Copyright 2023 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

import logging

import requests

from odoo import _, models
from odoo.exceptions import UserError

_logger = logging.getLogger(__name__)


class ApiConnector(models.AbstractModel):
    _name = "api.connector"
    _description = "API Connecteur"
    _backend_name = None

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
    def _get_backend(self):
        if self._backend_name:
            if self._backend_name == self._name:
                self.ensure_one()
                return self
            else:
                backend = self.env[self._backend_name].sudo().search([], limit=1)
                if backend:
                    return backend
                raise UserError(
                    _("You need to configure or active API backend: %s")
                    % (self._description,)
                )
        else:
            raise UserError(
                _("You need to configure API backend: %s") % (self._description,)
            )

    # ------------------------------------------------------
    # API functions
    # ------------------------------------------------------
    def call_api(self, url, call_type, **kwargs):
        """
        :param url: action API Alfresco
        :param call_type: post/get/put/delete
        @returns JSON response
        """
        _logger.info("Calling %s" % url)
        try:
            if call_type == "get":
                response = requests.get(url, **kwargs)
            elif call_type == "post":
                response = requests.post(url, **kwargs)
            elif call_type == "put":
                response = requests.put(url, **kwargs)
            elif call_type == "delete":
                response = requests.delete(url, **kwargs)
            response.raise_for_status()
        except requests.exceptions.ConnectionError as e:
            _logger.error("Connection to %s failed. Error: %s", url, e)
            raise e
        except requests.exceptions.RequestException as e:
            _logger.error("Error: %s", e)
            raise e
        except Exception as e:
            raise e

        return response.json()

    def api_get(self, url, **kwargs):
        return self.call_api(url, "get", **kwargs)

    def api_post(self, url, **kwargs):
        return self.call_api(url, "post", **kwargs)

    def api_put(self, url, **kwargs):
        return self.call_api(url, "put", **kwargs)

    def api_delete(self, url, **kwargs):
        return self.call_api(url, "delete", **kwargs)
